// Shader: /Shaders/LightingModels/Ambient.vsh
//

#version 110

#ifndef MAX_LIGHTS
#error Constant MAX_LIGHTS should be defined.
#endif

struct LightSource
{
    vec3 Position;
    float Radius;
    vec3 AmbientColor;
};

struct Material
{
    vec3 DiffuseColor;
};

uniform LightSource u_LightSource[MAX_LIGHTS];
uniform Material u_Material;

uniform vec3 u_AmbientColor;

varying vec3 out_N;
varying vec4 out_LnDistance[MAX_LIGHTS];

float CalcAttenuation( float Distance, float Radius )
{
    return clamp( 1.0 - pow( Distance / Radius, 2.0 ), 0.0, 1.0 );
}

float CalcSoftDiffuse( vec3 N, vec3 L, float Min )
{
    float Norm = dot( N, L ) * 0.5 + 0.5;
    float Diffuse = mix( Norm, 1.0, Min );

    return clamp( Diffuse, 0.0, 1.0 );
}

vec4 CalcFragColor()
{
    const float MIN_DIFFUSE_DOT_PRODUCT = 0.25;
    
    vec3 Color = u_AmbientColor * u_Material.DiffuseColor;  
    vec3 N = normalize( out_N );
    
    for (int i = 0; i < MAX_LIGHTS; ++i) 
    {
        vec3 L = normalize( out_LnDistance[i].xyz );
        float Distance = out_LnDistance[i].w;

        float Diffuse = CalcSoftDiffuse( N, L, MIN_DIFFUSE_DOT_PRODUCT );
        float Att = CalcAttenuation( Distance, u_LightSource[i].Radius ); 

        vec3 D = Diffuse * u_LightSource[i].AmbientColor * u_Material.DiffuseColor;

        Color += Att * D;
    }

    return vec4( Color, 0.0 );   
}

void main()
{
   gl_FragColor = CalcFragColor();
}