// Shader: /Shaders/Shadows/Wedge/Spherical/Construct.vsh
//

#version 110

struct LightSource
{
    vec3 Position;
#ifdef FRAG
    vec3 WorldPosition;
#endif
    float Radius;
    float ScaledRadius;
    float InvRadius;
};

uniform LightSource u_LightSource;

#ifdef FRAG
uniform mat4 u_ModelMatrix;
#endif

attribute vec3 in_Edge0;
attribute vec3 in_Edge1;
attribute vec2 in_VertexData;

#ifdef FRAG
varying vec4 out_Edge0;
varying vec4 out_Edge1;
#endif
                
void CalcAdjustedEdge( out vec3 Edge[2] )   
{
    vec3 Vec[2];
    vec2 Dist;

    Vec[0] = in_Edge0 - u_LightSource.Position;
    Dist[0] = dot( Vec[0], Vec[0] );
    Vec[1] = in_Edge1 - u_LightSource.Position;
    Dist[1] = dot( Vec[1], Vec[1] );
    
    vec2 Sq = sqrt( vec2( Dist[1] / Dist[0], Dist[0] / Dist[1] ) );
    if (Dist[0] > Dist[1]) 
    {
        Edge[0] = Vec[0] * Sq[0] + u_LightSource.Position;
        Edge[1] = in_Edge1;
    } 
    else 
    {      
        Edge[0] = Vec[1] * Sq[1] + u_LightSource.Position;
        Edge[1] = in_Edge0;
    } 
}

vec3 CalcWedgeVertex()
{
    vec3 Edge[2];

    CalcAdjustedEdge( Edge );
    if (in_VertexData.x == 1.0)
        return Edge[0];
    else if (in_VertexData.x == 2.0)
        return Edge[1];
    else 
    {
        vec3 Vecs[6];

        Vecs[0] = normalize( Edge[1] - Edge[0] );
    	Vecs[1] = cross( Vecs[0], u_LightSource.Position - Edge[0] );
    	Vecs[1] = normalize( Vecs[1] );
    	Vecs[2] = cross( Vecs[0], Vecs[1] );

    	Vecs[0] *= u_LightSource.Radius;
    	Vecs[1] *= u_LightSource.Radius;

        Vecs[3] = u_LightSource.Position + Vecs[2] * 1.0;//u_LightSource.ScaledRadius;
        Vecs[4] = Vecs[3] + Vecs[0];
        Vecs[5] = Vecs[3] - Vecs[0];
        
        if (in_VertexData.x == 0.0)
            return Edge[0] - (Vecs[4] + Vecs[1]);
        else if (in_VertexData.x == 3.0)
            return Edge[1] - (Vecs[5] + Vecs[1]);
        else if (in_VertexData.x == 4.0)
            return Edge[1] - (Vecs[5] - Vecs[1]);
        else // in_VertexData.x == 5.0
            return Edge[0] - (Vecs[4] - Vecs[1]);
    }
}

void main()
{
    vec3 Vertex = CalcWedgeVertex();
    
#ifdef FRAG
    out_Edge0 = u_ModelMatrix * vec4( in_Edge0, 1.0 );
    out_Edge1 = u_ModelMatrix * vec4( in_Edge1, 1.0 );
#endif

    gl_Position = gl_ModelViewProjectionMatrix * vec4( Vertex, in_VertexData.y ); 
}